#!/usr/bin/env python
# coding: utf-8

# In[1]:


#import packages 
from graphviz import Source
import graphviz
import pygraphviz as pgv
import networkx as nx
import matplotlib.pyplot as plt
import re
import glob
import csv
import collections
import time


# In[2]:


start_time = time.time() #return running time 


# # test Input and output with degree==1 and Angeli condition on SR-graph
# 
# 

# In[3]:


graphs_verified=[] # array with the graphs that verify the conditions 
path = '/Users/Lu/Downloads/e1/*.dot' #return the path of the graphs files to analyse
for files in glob.glob(path):
    with open(files) as path:
         
        m = re.search(r'/(.*?)__(.*?)__(.*?)__', files)
        
        #TO SELECT INPUT AND OUTPUT FROM THE FILE NAME
        input_n= m.group(2)
        output_n= m.group(3)
        
        s = Source.from_file(files)
        ###s.view() # to visualize the graph image 
        
        G = pgv.AGraph(files) 
        
        Grafo = nx.DiGraph() 
        
        for h in G.nodes(): #these methods build a DiGraph from an AGraph
            Grafo.add_node(h)
        
        for u,v in G.edges():
            Grafo.add_edge(u,v)
        
        
       
        degree_nodi=[]
        edge_set = set(tuple(edge) for edge in G.edges()) 

        in_edges = collections.defaultdict(list)
        out_edges = collections.defaultdict(list)

        for b,e in edge_set: #this method separate node-reactions from node-species
            if b.attr['shape'] == 'circle': #node-species are circle, otherwise the nodes represent reactions
                in_edges[e].append(b) #in_edges: reaction->species
            else:
                out_edges[b].append(e) #out_edges: reaction<-species

    
        
        #To build the SR-graph we have to remove all the reversible reactions
        #NB: both in_edges and out_edges are list reaction:species 
        reversible_reactions=[]
        for key1, value1 in in_edges.items():
            for key2, value2 in out_edges.items():
                if value2 == value1 and set(in_edges[key2]) == set(out_edges[key1]): #function set returns the key 
                    n2 = in_edges[key2]
                    n4 = out_edges[key2]

                    sum_value1= sum(1 for v in value1 if v) #check reversible edges if there are more than 1 reactants or product
                    sum_value2= sum(1 for v in value2 if v)
                    if sum_value1 >= 1 or sum_value2 >= 1: 
                        for d in n2:
                            for p in n4:
                                 reversible_reactions.append((p, key1, d)) #build reversible edges
       
        #remove reversible edges
        for j in reversible_reactions: 
            for z in reversible_reactions:
                if z[0] == j[2] and z[2] == j[0]:
                    reversible_reactions.remove(z)
        for i in reversible_reactions:
            if Grafo.has_edge(i[0], i[1]):
                Grafo.remove_edge(i[0], i[1])
            if Grafo.has_edge(i[1], i[2]):
                Grafo.remove_edge(i[1], i[2])
        
        
        for n in Grafo.nodes(): #check Angeli Condition on SR-graph
            
            if n.attr['shape'] == 'circle':
                degree = Grafo.degree(n) 
                
                if n==output_n and degree==1:
                    degree_nodi.append(1)
                elif n==output_n and degree!=1: 
                    degree_nodi.append(0)
                if n==input_n and degree==1:
                    degree_nodi.append(1)
                elif n==input_n and degree!=1: 
                    degree_nodi.append(2)
                if n!=output_n and n!= input_n and degree<=2:
                    degree_nodi.append(1)
                elif n!=output_n and n!= input_n and degree>2:
                    degree_nodi.append(3)
                    
                  
        if len(set(degree_nodi)) == 1 and degree_nodi[0]==1:
            graphs_verified.append(files) #save only the graphs that verify the condition
            
            
tempo = (time.time() - start_time)   #returns running time
if tempo <= 60:
    print (tempo, 'seconds')
else:
    tempo = tempo / 60
    print (tempo, 'minutes')


# In[4]:


print len(graphs_verified) #returns the number of graphs that verify the condition

with open('file_grafi_Cond_Angeli_condInOut_complete1.csv', mode='w') as csv_file: #write the name of graphs on a csv file
    fieldnames = ['grafi']
    writer = csv.DictWriter(csv_file, fieldnames=fieldnames)

    writer.writeheader()
    for s in graphs_verified:
        writer.writerow({'grafi': s})
